// /*
//  * Copyright (c) 2017 IpTronix S.r.l.
//  *
//  * SPDX-License-Identifier: Apache-2.0
//  */

#ifndef __INMP621_H__
#define __INMP621_H__

#include <zephyr/types.h>
#include <device.h>
#include <drivers/gpio.h>
#include <drivers/i2c.h>
// #include "nrfx_pdm.h"


/* User-programmable Registers */
#define INMP621_REG_I2C_CNTRL                 0x00
#define INMP621_REG_WOS_PGA_GAIN              0x01
#define INMP621_REG_WOS_FILTER                0x02
#define INMP621_REG_WOS_PGA_MIN_THR           0x03
#define INMP621_REG_WOS_PGA_MAX_THR           0x04
#define INMP621_REG_WOS_THRESHOLD             0x05

#define INMP621_DOUT_CLEAR_Pos               (4)
#define INMP621_WDT_DELAY_Pos             	(1)
#define INMP621_WOS_BANDPASS_HPF_Pos         (2)
#define INMP621_FAST_MODE_CNT_Pos            (5)
#define INMP621_WOS_RMS_Pos                  (5)

#define INMP621_DOUT_CLEAR_Msk               (0x01 << 4)
#define INMP621_WDT_DELAY_Msk             	(0x03 << 1)
#define INMP621_WDT_ENABLE_Msk               (0x01 << 0)

#define INMP621_WOS_PGA_GAIN_Msk             (0x1F << 0)
#define INMP621_WOS_BANDPASS_LPF_Msk         (0x03 << 0)
#define INMP621_WOS_BANDPASS_HPF_Msk         (0x03 << 2)

#define INMP621_FAST_MODE_CNT_Msk            (0x03 << 5)
#define INMP621_WOS_PGA_MIN_THR_Msk          (0x1F << 0)

#define INMP621_WOS_RMS_Msk                  (0x01 << 5)
#define INMP621_WOS_PGA_MAX_THR_Msk          (0x1F << 0)

#define INMP621_WOS_THRESHOLD_Msk            (0x07 << 0)

#if (CONFIG_INMP621_MONO)
#define INMP621_PDM_MODE NRF_PDM_MODE_MONO
#else
#define INMP621_PDM_MODE NRF_PDM_MODE_STEREO
#endif /* CONFIG_INMP621_MONO */

#if (CONFIG_INMP621_WDT_ENABLE)

#if defined(CONFIG_INMP621_WDT_DELAY_8_MS)
#define INMP621_WDT_DELAY		0x00
#elif defined(CONFIG_INMP621_WDT_DELAY_16_MS)
#define INMP621_WDT_DELAY		0x01
#elif defined(CONFIG_INMP621_WDT_DELAY_32_MS)
#define INMP621_WDT_DELAY		0x02
#elif defined(CONFIG_INMP621_WDT_DELAY_64_MS)
#define INMP621_WDT_DELAY		0x03
#endif

#else
#define INMP621_WDT_DELAY		0x00
#endif /* CONFIG_INMP621_WDT_ENABLE */

#if (CONFIG_INMP621_FAST_MODE_COUNT_ENABLE)

#if defined(CONFIG_INMP621_WINDOW_COMP_2_TRIPS)
#define INMP621_FAST_MODE_CNT		0x01
#elif defined(CONFIG_INMP621_WINDOW_COMP_4_TRIPS)
#define INMP621_FAST_MODE_CNT		0x02
#elif defined(CONFIG_INMP621_WINDOW_COMP_6_TRIPS)
#define INMP621_FAST_MODE_CNT		0x03
#endif

#else
/* 0x00 to disable */
#define INMP621_FAST_MODE_CNT		0x00
#endif /* CONFIG_INMP621_WDT_ENABLE */

#if defined(CONFIG_INMP621_LPF_2_KHZ)
#define INMP621_LPF_FREQUENCY		0x00
#elif defined(CONFIG_INMP621_LPF_4_KHZ)
#define INMP621_LPF_FREQUENCY		0x01
#elif defined(CONFIG_INMP621_LPF_6_KHZ)
#define INMP621_LPF_FREQUENCY		0x02
#elif defined(CONFIG_INMP621_LPF_8_KHZ)
#define INMP621_LPF_FREQUENCY		0x03
#else
#define INMP621_LPF_FREQUENCY		0x00
#endif

#if defined(CONFIG_INMP621_HPF_200_KHZ)
#define INMP621_HPF_FREQUENCY		0x00
#elif defined(CONFIG_INMP621_HPF_300_KHZ)
#define INMP621_HPF_FREQUENCY		0x01
#elif defined(CONFIG_INMP621_HPF_400_KHZ)
#define INMP621_HPF_FREQUENCY		0x02
#elif defined(CONFIG_INMP621_HPF_800_KHZ)
#define INMP621_HPF_FREQUENCY		0x03
#else
#define INMP621_HPF_FREQUENCY		0x03
#endif

/* Programmable Gain Amplifier gain LUT calculator */
#define INMP621_PGA_MAX_DB            91.5
#define INMP621_PGA_DB_STEPS          1.5
#define INMP621_PGA_TO_DB(x)          INMP621_PGA_MAX_DB - ((float)((x) & INMP621_WOS_PGA_GAIN_Msk) * INMP621_PGA_DB_STEPS)
#define INMP621_DB_TO_PGA(x)          (uint8_t)((INMP621_PGA_MAX_DB - (x)) / INMP621_PGA_DB_STEPS)   

#define INMP621_STANDBY_MODE_MIN 200000UL
#define INMP621_STANDBY_MODE_MAX 250000UL

#define INMP621_LOW_POWER_MODE_MIN 350000UL
#define INMP621_LOW_POWER_MODE_MAX 900000UL

#define INMP621_NORMAL_MODE_MIN 1000000UL
#define INMP621_NORMAL_MODE_MAX 3600000UL

typedef enum{
	ZPL,
	STANDBY,
	LOW_POWER,
	NORMAL
}dmic_op_mode;

typedef enum{
	HZ_4800 = 4800UL,
    HZ_6000 = 6000UL,
    HZ_8000 = 8000UL,
    HZ_9600 = 9600UL,
    HZ_11025 = 11025UL,
    HZ_12000 = 12000UL,
    HZ_16000 = 16000UL,
    HZ_32000 = 32000UL,
    HZ_44100 = 44100UL,
    HZ_48000 = 48000UL
}dmic_out_sample_rate;

struct inmp621_config {

    gpio_pin_t data_pin;
    gpio_pin_t clk_pin;
    bool lr_pin_level;

#if defined(CONFIG_INMP621_INT)
    const char *gpio_port;
    gpio_pin_t dout_pin;
    gpio_dt_flags_t dout_flags;
#endif
};

struct inmp621_data{

    dmic_op_mode inmp621_current_mode;
    nrfx_pdm_config_t inmp621_pdm_configs;
};

#endif /* __INMP621_H__ */
